<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

//
// NOTE:
//
// This instance of VOMSCompatibility implementation is based on PHP-SOAP
// in  contrast to manual XML creation
//
// Unfortunately, PHP SOAP is not compatible with AXIS 1.2 used in Java VOMS-Admin,
// only with 1.3 :-(
//
// Several problems arose while working on implementation: 
//   - WSDL contains overloaded methods which is no longer supported since WSDL 1.2
//   - PHP SOAP returns 'SOAP-ENV' in contrast to 'soapenv' prefix
//   - Response structure left the same, but node names differs
//
// Specification said that prefixes aren't required at all and node names are not 
// valuable in WSDL-mode. So if client implementation uses SOAP call - result will 
// be handled properly. But I'm not sure, that gLite implementations does not 
// use own XML parsers (especially for GET method) hardcoded to node names and so on.
// ( at least edg-mkgridmap uses hardcoded tag names to parse VOMS output )
//
// Looking at all this confusing things I have decided to stick on 'manual XML creation'
// that will produce output identical to Java VOMS-Admin as primary VOMSCompatibility 
// medthod, but include VOMSCompatibility2 into distribution to get available for 
// proper SOAP clients. WSDL has been pathched to eliminate overloaded methods.
//

	error_reporting(E_ALL); ini_set("display_errors", 1);
	ini_set("soap.wsdl_cache_enabled", "1");
	ini_set("include_path", "./interfaces:./modules:./lang:./conf");

	require("config.inc");
	require("get_vo_configuration.php");

	// WSDL URL
	$wsdl_url = 'http://'.$_SERVER['HTTP_HOST'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']);

	// return WSDL
	$wsdl_file = 'wsdl/voms-compatibility-2.0.3.wsdl';
	if ( isset($_GET['wsdl']) ) {
        	$wsdl = file_get_contents($wsdl_file);
	        echo preg_replace('/@@ENDPOINT@@/',$wsdl_url,$wsdl);
        	exit(0);
	}

	// define handler class
	class VOMSCompatibility {
		private $major_version = 0;
		private $minor_version = 0;
		private $patch_version = 0;

		function __construct ($wsdl_file) {
			$matches = array();
			if ( preg_match('/[^0-9]*([0-9])\.([0-9])\.([0-9])\.wsdl/',$wsdl_file, $matches) ) {
				$this->major_version = $matches[1];
				$this->minor_version = $matches[2];
				$this->patch_version = $matches[3];
			}
		}

		public function getMajorVersionNumber() {
			return $this->major_version;
		}

		public function getMinorVersionNumber() {
			return $this->minor_version;
		}

		public function getPatchVersionNumber() {
			return $this->patch_version;
		}

		public function getGridmapUsers($container = NULL) {
			if ( $container === NULL ) return getVOMembers();
			$gcr = getGroupCapabilityRole($container);
			return getVOContainerMembers($gcr[0], $gcr[2], $gcr[1]);
		}
	}


	// create SAOP server object
	$server = new SoapServer($wsdl_url.'?wsdl', array( 
			'uri' => $wsdl_url,
			'soap_version' => SOAP_1_1 )); // Java VOMS-Admin failed on 1.2 but I want to offer compatibility
	$server->setClass('VOMSCompatibility',$wsdl_file);

	// handle client request
	if ($_SERVER["REQUEST_METHOD"] == "POST") {
		$data = file_get_contents('php://input');
	} else {
		if ( isset($_GET['method']) ) {
			$method = $_GET['method'];
			$param = next($_GET);
			while (( $param == $vo ) || ( $param == $method ))
                         	$param = next($_GET);
		} else $server->fault('ns1:Client',"Error: method no specified");
		if ( $param ) $req = sprintf('<ns1:%s><param0 xsi:type="xsd:string">%s</param0></ns1:%s>', $method, $param, $method);
		else $req = sprintf('<ns1:%s/>',$method);

		$data = sprintf('<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://glite.org/wsdl/services/org.glite.security.voms.service.compatibility" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"><SOAP-ENV:Body>%s</SOAP-ENV:Body></SOAP-ENV:Envelope>', $req);
	}
	$server->handle($data);
?>
